@@ -7,7 +7,7 @@ use Mojo::Pg;
 use Sys::Hostname 'hostname';
 use Carp 'croak';
 
-our $VERSION = '4.0403'; ## copied from Minion::Backend::Pg of version 4.04
+our $VERSION = '5.00'; ## copied from Minion::Backend::Pg of version 5.00
 
 has 'pg';
 
@@ -28,9 +28,7 @@ sub dequeue {
 }
 
 sub enqueue {
-  my ($self, $task) = (shift, shift);
-  my $args    = shift // [];
-  my $options = shift // {};
+  my ($self, $task, $args, $options) = (shift, shift, shift || [], shift || {});
 
   my $db = $self->pg->db;
   return $db->query(
@@ -55,7 +53,7 @@ sub job_info {
      from minion_jobs where id = ?', shift
   )->hash;
   return undef unless $h;
-  $h->{args} = $h->{args} ? decode_json($h->{args}) : undef;
+  $h->{args} = $h->{args} ? decode_json($h->{args}) : [];
   $h->{result} = $h->{result} ? decode_json($h->{result}) : undef;
   return $h;
 }
@@ -67,8 +65,7 @@ sub list_jobs {
     'select id from minion_jobs
      where (state = $1 or $1 is null) and (task = $2 or $2 is null)
      order by id desc
-     limit $3
-     offset $4', @$options{qw(state task)}, $limit, $offset
+     limit $3 offset $4', @$options{qw(state task)}, $limit, $offset
   )->arrays->map(sub { $self->job_info($_->[0]) })->to_array;
 }
 
@@ -81,9 +78,9 @@ sub list_workers {
 }
 
 sub new {
-  my $self = shift->_is_pg91(@_);
-  croak 'PostgreSQL is not 9.1' unless $self;
-  $self = $self->SUPER::new(pg => Mojo::Pg->new(@_));
+  my $self = shift->SUPER::new(pg => Mojo::Pg->new(@_));
+  croak 'PostgreSQL 9.1 or later is required'
+    if Mojo::Pg->new(@_)->db->dbh->{pg_server_version} < 90100;
   my $pg = $self->pg->auto_migrate(1)->max_connections(1);
   $pg->migrations->name('minion')->from_data;
   return $self;
@@ -147,7 +144,7 @@ sub retry_job {
        retried = now(), retries = retries + 1, state = 'inactive',
        delayed = (now() + (interval '1 second' * ?))
      where id = ? and retries = ?
-       and state in ('failed', 'finished', 'inactive')
+       and state in ('inactive', 'failed', 'finished')
      returning 1", @$options{qw(priority queue)}, $options->{delay} // 0, $id,
     $retries
   )->rows;
@@ -156,6 +153,22 @@ sub retry_job {
 sub stats {
   my $self = shift;
 
+  my $stats = $self->pg->db->query(
+    "select state::text || '_jobs', count(state) from minion_jobs group by state
+     union all
+     select 'inactive_workers', count(*) from minion_workers
+     union all
+     select 'active_workers', count(distinct worker) from minion_jobs
+     where state = 'active'"
+  )->arrays->reduce(sub { $a->{$b->[0]} = $b->[1]; $a }, {});
+  $stats->{inactive_workers} -= $stats->{active_workers};
+  $stats->{"${_}_jobs"} ||= 0 for qw(inactive active failed finished);
+
+  return $stats;
+}
+
+sub _old_stats {
+  my $self = shift;
   my $db  = $self->pg->db;
   my $all = $db->query('select count(*) from minion_workers')->array->[0];
   my $sql
@@ -209,7 +222,7 @@ sub _try {
     $options->{queues} || ['default'], [keys %{$self->minion->tasks}]
   )->hash;
   return undef unless $h;
-  $h->{args} = $h->{args} ? decode_json($h->{args}) : undef;
+  $h->{args} = $h->{args} ? decode_json($h->{args}) : [];
   return $h;
 }
 
@@ -230,13 +243,6 @@ sub _update {
   return $self->retry_job($id, $retries, {delay => $delay});
 }
 
-sub _is_pg91 {
-    my $self = shift;
-    my $pg = Mojo::Pg->new(@_);
-    my $ver = $pg->db->query('SHOW server_version')->hash->{'server_version'};
-    return ($ver and $ver =~ /9\.1/i) ? $self : undef;
-}
-
 1;
 
 =encoding utf8
@@ -734,14 +740,22 @@ alter table minion_workers add column
   notified timestamp with time zone not null default now();
 alter table minion_workers alter column started set default now();
 
--- 3 up
-create index on minion_jobs (state);
-
 -- 4 up
 alter table minion_jobs add column queue text not null default 'default';
 
 -- 5 up
 alter table minion_jobs add column attempts int not null default 1;
 
--- 6 up
-drop index minion_jobs_state_idx;
+-- 7 up
+create type minion_state as enum ('inactive', 'active', 'failed', 'finished');
+alter table minion_jobs alter column state set default 'inactive'::minion_state;
+alter table minion_jobs
+  alter column state type minion_state using state::minion_state;
+
+-- 7 down
+alter table minion_jobs alter column state type text using state;
+alter table minion_jobs alter column state set default 'inactive';
+drop type if exists minion_state;
+
+-- 8 up
+create index on minion_jobs (state, priority desc, created);
